import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class SlideAnimation extends PageRouteBuilder {
  Widget widget;
  Duration duration = Duration(seconds: 3);
  SlideType type;

  SlideAnimation({@required this.widget, this.duration, this.type})
      : super(
          pageBuilder: (BuildContext context, Animation<double> animation,
              Animation<double> secondaryAnimation) {
            return widget;
          },
          transitionDuration: duration,
          transitionsBuilder: (BuildContext context,
              Animation<double> animation,
              Animation<double> secondaryAnimation,
              Widget child) {
            Animation<Offset> rightToLeft =
                Tween<Offset>(begin: Offset(1.0, 0.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> bottomToTop =
                Tween<Offset>(begin: Offset(0.0, 1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> leftToRight =
                Tween<Offset>(begin: Offset(-1.0, 0.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> topToBottom =
                Tween<Offset>(begin: Offset(0.0, -1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> rightBottomCorner =
                Tween<Offset>(begin: Offset(1.0, 1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> rightTopCorner =
                Tween<Offset>(begin: Offset(1.0, -1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> leftBottomCorner =
                Tween<Offset>(begin: Offset(-1.0, 1.0), end: Offset(0.0, 0.0))
                    .animate(animation);
            Animation<Offset> leftTopCorner =
                Tween<Offset>(begin: Offset(-1.0, -1.0), end: Offset(0.0, 0.0))
                    .animate(animation);

            return SlideTransition(
                position: type == SlideType.RTL
                    ? rightToLeft
                    : type == SlideType.LTR
                        ? leftToRight
                        : type == SlideType.TTB
                            ? topToBottom
                            : type == SlideType.BTT
                                ? bottomToTop
                                : type == SlideType.BRC
                                    ? rightBottomCorner
                                    : type == SlideType.BLC
                                        ? leftBottomCorner
                                        : type == SlideType.TRC
                                            ? rightTopCorner
                                            : type == SlideType.TLC
                                                ? leftTopCorner
                                                : rightToLeft,
                child: child);
          },
        );
}

/// Types of different slide transitions
/// [RTL] right to left
/// [LTR] left to right
/// [TTB] top to bottom
/// [BTT] bottom to top
/// [BRC] from bottom right corner
/// [BLC] from bottom left corner
/// [TRC] from top right corner
/// [TLC] from top left corner
enum SlideType {
  RTL,
  LTR,
  TTB,
  BTT,
  BRC,
  BLC,
  TRC,
  TLC,
}
